#
# See the file "LICENSE" for the full license governing this code.
#
module PuppetX
  module EnterpriseModules
    # Docs
    module OraInstall
      # Docs
      module GetHomes

        def default_database_sid
          Hash[*oratab_content.scan(/^(?!#)(.*):(.*):.*$/).flatten].keys.first
        end

        def oracle_home_for(sid)
          Hash[*oratab_content.scan(/^(?!#)(.*):(.*):.*$/).flatten].select { |k, _v| k == sid }.values.first
        end

        def group_for_file(home)
          #
          # To run opatch, we need the owner of the grid and oracle directory. The mechanism
          # we use to get this user, is to fetch the owner of the bin directory in the grid or
          # oracle home. Sometimes however, these directories also contain files with root as owner.
          # since root can never be the user of the grid or oracle processen, we skip those.
          #
          if os == 'windows'
            if File.exist?("#{get_orainst_loc}/oraInst.loc")
              file = File.read("#{get_orainst_loc}/oraInst.loc")
              file.scan(/inst_group=(.*)/).flatten.first
            end
          else
            a_file = first_non_root_file(home)
            if a_file
              gid = File.stat("#{home}/#{a_file}").gid
              Etc.getgrgid(gid).name
            else
              nil
            end
          end
        end

        def user_for_file(home)
          #
          # To run opatch, we need the owner of the grid and oracle directory. The mechanism
          # we use to get this user, is to fetch the owner of the bin directory in the grid or
          # oracle home. Sometimes however, these directories also contain files with root as owner.
          # since root can never be the user of the grid or oracle processen, we skip those.
          #
          if os == 'windows'
            group = group_for_file(home)
            users_in_group = `net localgroup #{group}`
            users = users_in_group.scan(/(^[a-zA-Z]*$)/).flatten
            users.reject!(&:empty?).reject! { |u| u == 'Members' }.first
          else
            a_file = first_non_root_file(home)
            if a_file
              uid = File.stat("#{home}/#{a_file}").uid
              Etc.getpwuid(uid).name
            else
              nil
            end
          end
        end

        def first_non_root_file(dir)
          if Dir.exist?(dir)
            Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
          else
            nil
          end
        end

        def os
          Facter.value(:kernel)
        end

        def get_homes(path)
          unless path.nil?
            if File.exist?(path + '/ContentsXML/inventory.xml')
              file = File.read(path + '/ContentsXML/inventory.xml')
              doc = REXML::Document.new(file)
              homes = []
              doc.elements.each('/INVENTORY/HOME_LIST/HOME') do |element|
                #
                # Skip all entryes that have subelements.
                #
                next if element.elements[1] && element.elements[1].name == 'DEPHOMELIST'
                str = element.attributes['LOC']
                name = element.attributes['NAME']
                if element.attributes['REMOVED'] == 'T'
                  Puppet.debug "Home #{str} marked as removed in Oracle inventory; skipping for ora_install_homes fact."
                  next
                end
                unless str.nil?
                  if str.include? 'plugins'
                    # skip EM agent
                  elsif str.include? 'jdk'
                    # skip EM agent
                  elsif str.include? 'OraPlaceHolderDummyHome'
                    # skip OraPlaceHolderDummyHome
                  else
                    homes << str
                  end
                end
              end
              return homes
            else
              return []
            end
          else
            return []
          end
        end

        def get_orainst_loc
          if os == 'windows'
            begin
              str = Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\\Oracle').read('inst_loc')[1]
              return str
            rescue
              return ''
            end
          else
            if File.exist?("#{ora_invdir}/oraInst.loc")
              str = ''
              output = File.read("#{ora_invdir}/oraInst.loc")
              output.split(/\r?\n/).each do |item|
                if item.match(/^inventory_loc/)
                  str = item[14, 50]
                end
              end
              return str
            else
              return ''
            end
          end
        end

        def oracle_homes(resources = nil)
          if resources
            resources.map { |_k, v| v.oracle_product_home_dir }.uniq
          else
            get_homes(get_orainst_loc)
          end
        end

        def installed_patches(resources = nil)
          if resources
            oracle_homes(resources).collect do |oracle_home|
              patches_in_home(oracle_home, ora_invdir, resources)
            end.flatten
          else
            oracle_homes.collect do |oracle_home|
              patches_in_home(oracle_home, ora_invdir)
            end.flatten
          end
        end

        def patches_in_home(oracle_product_home_dir, orainst_dir, resources = nil)
          if !resources
            os_user = user_for_file(oracle_product_home_dir)
            if os_user
              Dir.chdir(oracle_product_home_dir) do
                full_command = os == 'windows' ? \
                  "#{oracle_product_home_dir}\\OPatch\\opatch.bat lspatches -jdk #{oracle_product_home_dir}\\jdk -oh #{oracle_product_home_dir}" : \
                  "#{oracle_product_home_dir}/OPatch/opatch lspatches -jdk #{oracle_product_home_dir}/jdk -oh #{oracle_product_home_dir} -invPtrLoc #{orainst_dir}/oraInst.loc"
                raw_list = Puppet::Util::Execution.execute(full_command, :failonfail => true, :uid => os_user, :combine => true )
                Puppet.debug "\n#{raw_list}"
                cleanup_opatch_residue([oracle_product_home_dir])
                raw_list.scan(/^(\d+);(.*)$/).collect { |e| { :name => "#{oracle_product_home_dir}:#{e[0]}", :comment => e[1] } }
              end
            else
              []
            end
          else
            ora_install_homes = Facter.value(:ora_install_homes) || {}
            patches = ora_install_homes.dig('installed_patches', oracle_product_home_dir)
            patches.nil? ? [] : patches.collect { |p| { :name => "#{oracle_product_home_dir}:#{p}", :comment => '', :ensure => :present } }
          end
        end

        def cleanup_opatch_residue(homes)
          # TODO: This is duplicate code from facts.rb. We need to make this more common
          # Some older versions of Opatch leave files and directories arround
          # When running opatch often, this causes some issues. This is to make
          # sure all files created by running Opatch as part of the fact, are remopved
          basename = Time.now.strftime("%Y-%m-%d_%H-%M-*%p")
          FileUtils.rm_f("/OraInstall2018#{basename}")
          homes.each {|home| FileUtils.rm_f("#{home}/cfgtoollogs/opatch/lsinv/lsinventory#{basename}.txt") }
        end

        def oratab_exists?
          if os == 'windows'
            false
          else
            File.exist?(oratab_file)
          end
        end

        def oratab_content
          if File.exist?(oratab_file)
            File.read(oratab_file)
          else
            ''
          end
        end

        def oratab_file
          "#{ora_invdir}/oratab"
        end

        def ora_invdir
          case os
          when 'Linux' then
            '/etc'
          when 'SunOS' then
            '/var/opt/oracle'
          when 'windows' then
            get_orainst_loc
          else
            '/etc'
          end
        end
      end
    end
  end
end
